# type: ignore[attr-defined]
import typer
import subprocess
import time
import ast
import re
import os
from typing import Optional
import json
from collections import OrderedDict
from rich.console import Console
from typing import List
import boto3
import awstk.db as db
from pathlib import Path
import shutil
from botocore.exceptions import ClientError

app = typer.Typer()
font_color = 'red'
console = Console()
cfn = boto3.resource('cloudformation', endpoint_url=os.environ.get("AWSTK_AWS_ENDPOINT_URL", None))
_db = db.get_connection(tablename='cloudformation')
dirprefix = Path.home() / '.awstk' / 'cloudformation'
valid_status = ['CREATE_COMPLETE', 'ROLLBACK_FAILED', 'ROLLBACK_COMPLETE', 'UPDATE_COMPLETE_CLEANUP_IN_PROGRESS', 'UPDATE_COMPLETE',
                'UPDATE_ROLLBACK_FAILED', 'UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS', 'UPDATE_ROLLBACK_COMPLETE', 'REVIEW_IN_PROGRESS']


@app.command()
def mirror(target_stack_regex_list: List[str]):
    console.print(
        f"[{font_color}]{'Cloudformation: Mirroring non-updating stacks into local database..'}[/]")

    all_stacks = [stack for stack in cfn.stacks.all(
    ) if stack.stack_status in valid_status]

    stacks = []
    if target_stack_regex_list:
        for stack in all_stacks:
            is_valid_stack = any([re.search(target_stack_regex, stack.name)
                                 for target_stack_regex in target_stack_regex_list])
            if is_valid_stack:
                stacks.append(stack)

    if not stacks:
        console.print(
            f"[{font_color}]{'No stacks found. Exiting..'}[/]")
        return

    for s in stacks:
        j = cfn.meta.client.get_template(StackName=s.name)['TemplateBody']
        if type(j) == str:
            j = str(j)
        elif type(j) == OrderedDict:
            j = str(json.dumps(j))
        else:
            raise ValueError

        console.print(f"[{font_color}]{s.name}[/] done.")
        _db[s.name] = {'templatebody': j,
                       'parameters': json.dumps(s.parameters)}
        _db.commit()
        time.sleep(1)

@app.command()
def list(local: bool = False):
    if local:
        all_stacks = _db.keys()
    else:
        all_stacks = [stack.name for stack in cfn.stacks.all() if stack.stack_status in valid_status]

    for stackname in all_stacks:
        console.print(stackname)
        

@ app.command()
def edit(target_stack_name: str, edit_immediately: bool = True):
    dirpath = dirprefix / target_stack_name

    if dirpath.exists() and dirpath.is_dir():
        shutil.rmtree(dirpath)
    os.makedirs(dirpath)
    f_parameters = open(dirpath / 'parameters.yaml', "w")
    f_templatebody = open(dirpath / 'template.yaml', "w")

    try:
        blob = _db[target_stack_name]

        f_parameters.write(blob['parameters'])
        f_templatebody.write(blob['templatebody'])

    except KeyError:
        console.print(f"[{font_color}] Stack Not Found. Available stacks: [/]")
        for key in _db.keys():
            console.print(key)
        console.print(f"[{font_color}] Creating template stack files.. [/]")
        f_parameters.write(
            '[{"ParameterKey": "SampleParameter", "ParameterValue": "SampleValue"}]')
        f_templatebody.write('Resources:')

    f_parameters.close()
    f_templatebody.close()
    console.print(f"[{font_color}]Stack ready to edit at {dirpath}[/]")

    if edit_immediately:
        subprocess.run([os.environ.get("EDITOR", "vim"), dirpath])


@ app.command()
def save(target_stack_name: str):
    try:
        dirpath = dirprefix / target_stack_name

        f_parameters = open(dirpath / 'parameters.yaml', "r").read()
        f_templatebody = open(dirpath / 'template.yaml', "r").read()

        cfn.meta.client.validate_template(TemplateBody=f_templatebody)

        _db[target_stack_name] = {'templatebody': f_templatebody,
                                  'parameters': f_parameters}

        _db.commit()
        console.print(f"[{font_color}]Stack successfully saved.[/]")

    except FileNotFoundError:
        console.print(f"[{font_color}] Stack files not found in {dirpath}.[/]")


@ app.command()
def update(target_stack_name: str, use_change_set: bool = True):
    try:
        tb = _db[target_stack_name]['templatebody']
        p = _db[target_stack_name]['parameters']
        C = ['CAPABILITY_IAM', 'CAPABILITY_NAMED_IAM', 'CAPABILITY_AUTO_EXPAND']
        if p == 'null':
            p = '[]'

        if use_change_set:
            cfn.meta.client.create_change_set(
                StackName=target_stack_name, TemplateBody=tb, Parameters=ast.literal_eval(p), Capabilities=C, ChangeSetName=target_stack_name)
            console.print(
                f"[{font_color}]Changeset created. Check AWS Cloudformation UI for review.[/]")
        else:
            cfn.meta.client.update_stack(
                StackName=target_stack_name, TemplateBody=tb, Parameters=ast.literal_eval(p), Capabilities=C)
            console.print(
                f"[{font_color}]Stack update initiated. Check AWS Cloudformation UI for any updates.[/]")
    except KeyError:
        console.print(f"[{font_color}] Stack Not Found. Available stacks: [/]")
        for key in _db.keys():
            console.print(key)
        return
    except ClientError as e:
        if "does not exist" in str(e):
            cfn.meta.client.create_stack(
                StackName=target_stack_name, TemplateBody=tb, Parameters=ast.literal_eval(p), Capabilities=C)
            console.print(
                f"[{font_color}]Stack didn't exist and is being created. Check AWS Cloudformation UI for any updates.[/]")
        else:
            raise e


if __name__ == "__main__":
    app()



import json


class DashboardFactory:
    def __init__(self, time_range: str = '-P3H'):
        self.widget_template = {'start': time_range, 'widgets': []}

    def get_dashboard_body(self):
        return json.dumps(self.widget_template)

    def _add_metric_widget(self, x: int, y: int, instance_id: str, label: str, namespace: str, metric: str, width: int = 24, height: int = 6):
        widget = {
            "type": "metric",
            "x": x,
            "y": y,
            "width": width,
            "height": height,
            "properties": {
                "region": "ap-southeast-1",
                "view": "timeSeries",
                "stacked": False,
                "metrics": [[namespace, metric, "InstanceId", instance_id]],
                "title": label}
        }
        return widget

    def add_cpu_widget(self, x: int, y: int, instance_id: str, label: str, width: int = 24, height: int = 6, cw_namespace: str = "AWS/EC2"):
        widget = self._add_metric_widget(
            x, y, instance_id, label, cw_namespace, "CPUUtilization", width, height)
        self.widget_template['widgets'].append(widget)

    def add_mem_widget(self, x: int, y: int, instance_id: str, label: str, width: int = 24, height: int = 6, cw_namespace: str = 'CWAgent'):
        widget = self._add_metric_widget(
            x, y, instance_id, label, cw_namespace, "Memory % Committed Bytes In Use", width, height)
        self.widget_template['widgets'].append(widget)

    def add_disk_space_widget(self, x: int, y: int, instance_id: str, label: str, width: int = 24, height: int = 6, cw_namespace: str = 'CWAgent'):
        widget = self._add_metric_widget(
            x, y, instance_id, label, cw_namespace, "LogicalDisk % Free Space", width, height)
        self.widget_template['widgets'].append(widget)



from sqlitedict import SqliteDict
import os

_connection = None


def get_connection(tablename: str):
    global _connection
    if not _connection:
        _connection = SqliteDict(
            f"{os.path.expanduser('~')}/.awstk.db", tablename=tablename)
    return _connection



# type: ignore[attr-defined]
import typer
from rich.console import Console

import awstk.cloudformation as cloudformation
import awstk.secretsmanager as secretsmanager
import awstk.cloudwatch as cloudwatch
import awstk.closest_match as closest_match


app = typer.Typer(name="awstk", add_completion=True)
app.add_typer(cloudformation.app, name="cloudformation")
app.add_typer(secretsmanager.app, name="secretsmanager")
app.add_typer(cloudwatch.app, name="cloudwatch")
app.add_typer(closest_match.app, name="closest-match")
console = Console()

if __name__ == "__main__":
    app()



# type: ignore[attr-defined]
import typer
import mimetypes
import difflib
import os
from pathlib import Path
from typing import List
from rich.console import Console

app = typer.Typer()
font_color = 'green'
console = Console()


@app.command()
def find(target_file: Path, target_directories: List[Path]):
    console.print(
        f"[{font_color}]Finding closest matching file {target_file}... [/]")
    if not Path.exists(target_file):
        console.print(f"[{font_color}]{target_file} does not exist. [/]")
        return
    if any([not os.path.isdir(target_directory) for target_directory in target_directories]):
        console.print(
            f"[{font_color}]One or more of directories does not exist. [/]")
        return

    dirs = []
    for target_directory in target_directories:
        dirs += [path for path in target_directory.glob(
            f"**") if not any(part.startswith('.') for part in path.parts)]

    fpaths = []
    for d in dirs:
        files = os.listdir(d)
        fpaths += [os.path.join(d, f)
                   for f in files if os.path.isfile(os.path.join(d, f)) and not f.startswith('.')]
    f_txt = [f for f in fpaths if mimetypes.guess_type(
        f)[0] == None or 'text' in mimetypes.guess_type(f)[0]]

    f_txt_with_wc = []
    for txt in f_txt:
        console.print(f"[{font_color}]Checking {txt}..[/]")
        target_file_contents = open(
            target_file, 'r').read().splitlines(keepends=True)
        cmp_file_contents = open(txt, 'r').read().splitlines(keepends=True)
        raw_diff = list(difflib.ndiff(target_file_contents, cmp_file_contents))
        diff = [d for d in raw_diff if d.startswith('+') or d.startswith('-')]
        wc = len(diff)
        f_txt_with_wc += [(txt, wc)]

    results = sorted(f_txt_with_wc, key=lambda tup: tup[1])
    for result in results:
        console.print(f"[{font_color}]{result[0]}:{result[1]}[/]")


if __name__ == "__main__":
    app()



# type: ignore[attr-defined]
import typer
import subprocess
import json
import os
import shutil
from pathlib import Path
from rich.console import Console
import boto3
from botocore.exceptions import ClientError

app = typer.Typer()
font_color = 'blue'
console = Console()
smgr_client = boto3.client('secretsmanager', endpoint_url=os.environ.get("AWSTK_AWS_ENDPOINT_URL", None))
dirprefix = Path.home() / '.awstk' / 'secretsmanager'


@app.command()
def edit(target_secret_name: str, edit_immediately: bool = True):
    sanitized_secret_name = target_secret_name.replace('/', '.')
    dirpath = dirprefix / sanitized_secret_name

    if dirpath.exists() and dirpath.is_dir():
        shutil.rmtree(dirpath)
    os.makedirs(dirpath)
    f_secret_string = open(dirpath / 'secret_string.yaml', "w")

    try:
        secret = json.loads(smgr_client.get_secret_value(
            SecretId=target_secret_name)['SecretString'])
        json.dump(secret, f_secret_string, indent=4)
    except smgr_client.exceptions.ResourceNotFoundException:
        console.print(
            f"[{font_color}]Secret doesn't currently exist. Creating template file.[/]")
        json.dump({'key': 'value'}, f_secret_string, indent=4)

    f_secret_string.close()

    console.print(f"[{font_color}]Secret ready to edit at {dirpath}[/]")

    if edit_immediately:
        subprocess.run([os.environ.get("EDITOR", "vim"), dirpath])


@app.command()
def update(target_secret_name: str):
    try:
        sanitized_secret_name = target_secret_name.replace('/', '.')
        dirpath = dirprefix / sanitized_secret_name

        f_secret_string = open(dirpath / 'secret_string.yaml', "r").read()

        smgr_client.update_secret(SecretId=target_secret_name,
                                  SecretString=f_secret_string)
        console.print(f"[{font_color}]Secret successfully updated.[/]")

    except FileNotFoundError:
        console.print(f"[{font_color}] Secret not found in {dirpath}.[/]")
        return

    except json.JSONDecodeError as err:
        console.print(f"Malformed secret at {dirpath}")
        start, stop = max(0, err.pos - 20), err.pos + 20
        snippet = err.doc[start:stop]
        console.print('... ' if start else '', snippet,
                      ' ...' if stop < len(err.doc) else '', sep="")
        console.print('^'.rjust(21 if not start else 25))
        return

    except ClientError as e:
        if "can't find the specified secret" in str(e):
            smgr_client.create_secret(Name=target_secret_name,
                                      SecretString=f_secret_string)
            console.print(
                f"[{font_color}]Secret didn't exist before, successfully created.[/]")
        else:
            raise e

    shutil.rmtree(dirpath)
    console.print(f"[{font_color}]Secret removed from {dirpath}.[/]")


if __name__ == "__main__":
    app()



# type: ignore[attr-defined]
import typer
import os
import re
import boto3
from rich.console import Console
from pathlib import Path
from awstk.dashboard import DashboardFactory

app = typer.Typer()
console = Console()
dirprefix = Path.home() / '.awstk' / 'cloudwatch'
ec2 = boto3.resource('ec2', endpoint_url=os.environ.get("AWSTK_AWS_ENDPOINT_URL", None))
cw_client = boto3.client('cloudwatch', endpoint_url=os.environ.get("AWSTK_AWS_ENDPOINT_URL", None))
logs_client = boto3.client('logs', endpoint_url=os.environ.get("AWSTK_AWS_ENDPOINT_URL", None))


@app.command(name="list-ec2")
def list_ec2(tag_key: str = typer.Argument(...),
             tag_value_regex: str = typer.Argument(...),
             ec2_status: str = typer.Argument('running'),
             print_stdout: bool = typer.Option(True)):

    try:
        instances = [i for i in ec2.instances.all() if i.tags is not None and any(
            [x for x in i.tags if tag_key == x['Key'] and re.search(tag_value_regex, x['Value']) and i.state['Name'] == ec2_status])]
    except re.error:
        return

    if print_stdout:
        for instance in instances:
            keyValue = [x['Value']
                        for x in instance.tags if x['Key'] == tag_key][0]
            console.print(f"{keyValue},{instance.id}")

    return instances


@ app.command(name="generate-dashboard")
def generate_dashboard(
    tag_key: str = typer.Argument(...),
    tag_value_regex: str = typer.Argument(...),
    cpu: bool = typer.Option(False),
    mem: bool = typer.Option(False),
    disk: bool = typer.Option(False),
    log: bool = typer.Option(False),
    widget_height: int = typer.Option(6),
    widget_width: int = typer.Option(24),
    time_range: str = typer.Option('-P3H')


):
    instances = list_ec2(tag_key, tag_value_regex,
                         print_stdout=False, ec2_status='running')

    if not instances:
        console.print(f"[bold {'yellow'}]{'No EC2 instances were found.'}[/]")
        return

    if cpu:
        dashboard = DashboardFactory(time_range=time_range)
        console.print(
            f"[bold {'yellow'}]{'Generating CPUUsage Dashboard..'}[/]")
        for i, instance in enumerate(instances):
            label = [x for x in instance.tags if x['Key']
                     == tag_key][0]['Value']
            dashboard.add_cpu_widget(
                x=0, y=widget_height*i,
                instance_id=instance.id, label=label, width=widget_width, height=widget_height)

        dashboard_body = dashboard.get_dashboard_body()
        cw_client.put_dashboard(DashboardName='CPUUsage',
                                DashboardBody=dashboard_body)
        console.print(f"[bold {'yellow'}]{'CPUUsage Dashboard generated.'}[/]")

    if mem:
        dashboard = DashboardFactory(time_range=time_range)
        console.print(
            f"[bold {'yellow'}]{'Generating MemUsage Dashboard..'}[/]")
        for i, instance in enumerate(instances):
            label = [x for x in instance.tags if x['Key']
                     == tag_key][0]['Value']
            dashboard.add_mem_widget(
                x=0, y=widget_height*i,
                instance_id=instance.id, label=label, width=widget_width, height=widget_height, cw_namespace='CWAgent')

        dashboard_body = dashboard.get_dashboard_body()
        cw_client.put_dashboard(DashboardName='MemUsage',
                                DashboardBody=dashboard_body)
        console.print(f"[bold {'yellow'}]{'MemUsage Dashboard generated.'}[/]")

    if disk:
        dashboard = DashboardFactory(time_range=time_range)
        console.print(
            f"[bold {'yellow'}]{'Generating DiskUsage Dashboard..'}[/]")
        for i, instance in enumerate(instances):
            label = [x for x in instance.tags if x['Key']
                     == tag_key][0]['Value']
            dashboard.add_disk_space_widget(
                x=0, y=widget_height*i,
                instance_id=instance.id, label=label, width=widget_width, height=widget_height, cw_namespace='CWAgent')

        dashboard_body = dashboard.get_dashboard_body()
        cw_client.put_dashboard(DashboardName='DiskUsage',
                                DashboardBody=dashboard_body)
        console.print(
            f"[bold {'yellow'}]{'DiskUsage Dashboard generated.'}[/]")


if __name__ == "__main__":
    app()
